@@ -59,9 +59,9 @@ static int C_matchClass(SEXP obj, SEXP classes);
 static void C_traverse(SEXP X, CountGlobal *count, int depth);
 static SEXP C_eval_list(SEXP env, SEXP Xi, SEXP fcall, SEXP pcall, SEXP classes, SEXP deflt, SEXP xsym, SEXP *xparents, SEXP xsiblings, PROTECT_INDEX ipx, Args args, CountGlobal *countglobal, CountLocal countlocal, R_len_t (**xinfo)[3], R_len_t **xloc, R_len_t **xdepth);
 static SEXP C_fill_list(SEXP Xi, R_len_t (*xinfo)[3], R_len_t *buf, R_len_t node, R_len_t maxnodes, R_len_t ibuf);
-static void C_fill_flat(SEXP ansNew, SEXP Xi, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians);
-static void C_fill_flat_names(SEXP ansNew, SEXP newNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians);
-static void C_fill_melt(SEXP ansFlat, SEXP ansNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians);
+static void C_fill_flat(SEXP ansNew, SEXP Xi, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians, R_len_t maxnodes, int dfaslist);
+static void C_fill_flat_names(SEXP ansNew, SEXP newNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians, R_len_t maxnodes, int dfaslist);
+static void C_fill_melt(SEXP ansFlat, SEXP ansNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians, R_len_t maxnodes, int dfaslist);
 SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPred, SEXP classes, SEXP how, SEXP deflt, SEXP R_dfaslist, SEXP R_feverywhere);
 /* ---------------------- */
 
@@ -88,7 +88,7 @@ SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPre
 	R_args.pxparents = INTEGER_ELT(argsPred, 2) > 0;
 	R_args.pxsiblings = INTEGER_ELT(argsPred, 3) > 0;
 	R_args.dfaslist = LOGICAL_ELT(R_dfaslist, 0);
-	R_args.feverywhere = INTEGER_ELT(R_feverywhere, 0) - 1;
+	R_args.feverywhere = INTEGER_ELT(R_feverywhere, 0);
 
 	/* traverse list once for max nodes and max depth
 	   for more accurate initialization, computational 
@@ -165,7 +165,7 @@ SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPre
 		if (R_args.fxparents)
 		{
 			fCDR = CDR(fCDR);
-			SET_TAG(CDR(fCDR), xparents);
+			SET_TAG(fCDR, xparents);
 		}
 		if (R_args.fxsiblings)
 		{
@@ -219,7 +219,7 @@ SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPre
 		if (R_args.pxparents)
 		{
 			pCDR = CDR(pCDR);
-			SET_TAG(CDR(pCDR), xparents);
+			SET_TAG(pCDR, xparents);
 		}
 		if (R_args.pxsiblings)
 		{
@@ -269,15 +269,6 @@ SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPre
 		/* update current node info for list pruning */
 		if (R_args.how_C > 2)
 		{
-			/* reallocate array if necessary in this case */
-			if (R_args.feverywhere == 2 && (initGlobal.node + 1) >= initGlobal.maxnodes)
-			{
-				xinfo = (R_len_t(*)[3])S_realloc((char *)xinfo, 2 * initGlobal.maxnodes, initGlobal.maxnodes, sizeof(*xinfo));
-				if (R_args.how_C == 5)
-					xdepth = (R_len_t *)S_realloc((char *)xinfo, 2 * initGlobal.maxnodes, initGlobal.maxnodes, sizeof(R_len_t));
-				initGlobal.maxnodes *= 2;
-			}
-
 			initGlobal.node++;				// increment node counter
 			xinfo[initGlobal.node][1] = -1; // parent node counter
 			xinfo[initGlobal.node][2] = i;	// child node counter
@@ -361,19 +352,14 @@ SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPre
 			if (!initGlobal.anynames)
 			{
 				for (R_len_t i = 0; i < Rf_length(ans); i++)
-				{
-					C_fill_flat(ansNew, VECTOR_ELT(ans, i), xinfo, &ix, &ians);
-					ix++;
-				}
+					C_fill_flat(ansNew, VECTOR_ELT(ans, i), xinfo, &ix, &ians, initGlobal.maxnodes, R_args.dfaslist);
 			}
 			else
 			{
 				SEXP newNames = PROTECT(Rf_allocVector(STRSXP, m));
 				for (R_len_t i = 0; i < Rf_length(ans); i++)
-				{
-					C_fill_flat_names(ansNew, newNames, VECTOR_ELT(ans, i), Rf_isNull(names) ? NA_STRING : STRING_ELT(names, i), xinfo, &ix, &ians);
-					ix++;
-				}
+					C_fill_flat_names(ansNew, newNames, VECTOR_ELT(ans, i), Rf_isNull(names) ? NA_STRING : STRING_ELT(names, i), xinfo, &ix, &ians, initGlobal.maxnodes, R_args.dfaslist);
+
 				Rf_setAttrib(ansNew, R_NamesSymbol, newNames);
 				UNPROTECT(1);
 			}
@@ -401,8 +387,7 @@ SEXP C_rrapply(SEXP env, SEXP X, SEXP FUN, SEXP argsFun, SEXP PRED, SEXP argsPre
 				if (noNames)
 					SET_STRING_ELT(namesNew, i, C_int2char(i + 1));
 
-				C_fill_melt(ansFlat, ansNames, VECTOR_ELT(ans, i), STRING_ELT(namesNew, i), xinfo, &ix, &ians);
-				ix++;
+				C_fill_melt(ansFlat, ansNames, VECTOR_ELT(ans, i), STRING_ELT(namesNew, i), xinfo, &ix, &ians, initGlobal.maxnodes, R_args.dfaslist);
 			}
 
 			/* add flat list to data.frame */
@@ -491,7 +476,7 @@ static int C_matchClass(SEXP obj, SEXP classes)
 		for (R_len_t i = 0; i < n; i++)
 			for (R_len_t j = 0; j < Rf_length(classes); j++)
 				if (strcmp(CHAR(STRING_ELT(klass, i)), CHAR(STRING_ELT(classes, j))) == 0)
-					matched = 1;
+					matched = TRUE;
 	}
 	else
 	{
@@ -518,7 +503,6 @@ static int C_matchClass(SEXP obj, SEXP classes)
 		{
 			SEXPTYPE type = (SEXPTYPE)TYPEOF(obj);
 			const char *typename;
-			/* excluded LANGSXP type */
 			switch (type)
 			{
 			case CLOSXP:
@@ -532,6 +516,15 @@ static int C_matchClass(SEXP obj, SEXP classes)
 			case SYMSXP:
 				typename = "name";
 				break;
+			case EXPRSXP:
+				typename = "expression";
+				break;
+			case LISTSXP:
+				typename = "pairlist";
+				break;
+			case LANGSXP:
+				typename = "language";
+				break;
 			default:
 				typename = CHAR(Rf_type2str(type));
 			}
@@ -659,7 +652,7 @@ static SEXP C_eval_list(
 		/* update current .xsiblings value */
 		if (args.fxsiblings || args.pxsiblings)
 		{
-			if(Rf_isPairList(xsiblings))
+			if (Rf_isPairList(xsiblings))
 			{
 				R_len_t n = Rf_length(xsiblings);
 				xsiblings_val = PROTECT(Rf_allocVector(VECSXP, n));
@@ -673,7 +666,7 @@ static SEXP C_eval_list(
 				Rf_setAttrib(xsiblings_val, R_NamesSymbol, PROTECT(Rf_getAttrib(xsiblings, R_NamesSymbol)));
 				UNPROTECT(1);
 			}
-			else 
+			else
 			{
 				xsiblings_val = PROTECT(Rf_duplicate(xsiblings));
 			}
@@ -753,7 +746,7 @@ static SEXP C_eval_list(
 		else
 			matched = C_matchClass(Xi, classes);
 
-		if (args.pArgs > 0 && !skip)
+		if (matched && !skip && args.pArgs > 0)
 		{
 			/* set default to FALSE */
 			doEval = FALSE;
@@ -773,7 +766,7 @@ static SEXP C_eval_list(
 		}
 
 		/* evaluate f and decide what to return or recurse further */
-		if (doEval && matched && !skip)
+		if (matched && !skip && doEval)
 		{
 			/* update current node info only for pruning and melting */
 			if (args.how_C > 2)
@@ -787,7 +780,7 @@ static SEXP C_eval_list(
 					if (i2 > -1)
 					{
 						i1 = i2;
-						(*xinfo)[i1][0] = 2;
+						(*xinfo)[i1][0] = 2; // non-terminal node
 						i2 = (*xinfo)[i1][1];
 					}
 					else
@@ -901,18 +894,6 @@ static SEXP C_eval_list(
 			/* update current node info */
 			if (args.how_C > 2)
 			{
-				/* reallocate arrays if necessary in this case */
-				if (args.feverywhere == 2)
-				{
-					if ((countglobal->node + 1) >= countglobal->maxnodes)
-					{
-						*xinfo = (R_len_t(*)[3])S_realloc((char *)*xinfo, 2 * countglobal->maxnodes, countglobal->maxnodes, sizeof(**xinfo));
-						if (args.how_C == 5)
-							*xdepth = (R_len_t *)S_realloc((char *)*xdepth, 2 * countglobal->maxnodes, countglobal->maxnodes, sizeof(R_len_t));
-						countglobal->maxnodes *= 2;
-					}
-				}
-
 				countglobal->node += 1;
 				(*xinfo)[countglobal->node][1] = countlocal.parent; /* parent node */
 				(*xinfo)[countglobal->node][2] = j;					/* child counter */
@@ -978,7 +959,7 @@ static SEXP C_fill_list(SEXP Xi, R_len_t (*xinfo)[3], R_len_t *buf, R_len_t node
 			if (xinfo[inode][1] < node || xinfo[inode][1] > maxparent)
 				break;
 			/* update maximum allowed parent node */
-			maxparent += 1;
+			maxparent++;
 		}
 
 		/* descend one level */
@@ -1023,27 +1004,38 @@ static SEXP C_fill_list(SEXP Xi, R_len_t (*xinfo)[3], R_len_t *buf, R_len_t node
 }
 
 /* fill flat list without names */
-static void C_fill_flat(SEXP ansNew, SEXP Xi, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians)
+static void C_fill_flat(SEXP ansNew, SEXP Xi, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians, R_len_t maxnodes, int dfaslist)
 {
-	if (xinfo[*ix][0] == 1)
+	(*ix)++;					// increment counter
+	if (xinfo[*ix - 1][0] == 1) // terminal node
 	{
 		SET_VECTOR_ELT(ansNew, *ians, Xi);
 		(*ians)++;
 	}
 	else if (Rf_isVectorList(Xi))
 	{
-		for (R_len_t i = 0; i < Rf_length(Xi); i++)
+		/* check if object is data.frame */
+		int doRecurse = TRUE;
+		if (!dfaslist)
 		{
-			(*ix)++;
-			C_fill_flat(ansNew, VECTOR_ELT(Xi, i), xinfo, ix, ians);
+			SEXP df = PROTECT(Rf_ScalarString(Rf_mkChar("data.frame")));
+			if (C_matchClass(Xi, df))
+				doRecurse = FALSE;
+			UNPROTECT(1);
 		}
+
+		/* recurse deeper */
+		if (doRecurse)
+			for (R_len_t i = 0; i < Rf_length(Xi); i++)
+				C_fill_flat(ansNew, VECTOR_ELT(Xi, i), xinfo, ix, ians, maxnodes, dfaslist);
 	}
 }
 
 /* fill flat list with names */
-static void C_fill_flat_names(SEXP ansNew, SEXP newNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians)
+static void C_fill_flat_names(SEXP ansNew, SEXP newNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians, R_len_t maxnodes, int dfaslist)
 {
-	if (xinfo[*ix][0] == 1)
+	(*ix)++;					// increment counter
+	if (xinfo[*ix - 1][0] == 1) // terminal node
 	{
 		SET_VECTOR_ELT(ansNew, *ians, Xi);
 		SET_STRING_ELT(newNames, *ians, name);
@@ -1051,48 +1043,73 @@ static void C_fill_flat_names(SEXP ansNew, SEXP newNames, SEXP Xi, SEXP name, R_
 	}
 	else if (Rf_isVectorList(Xi))
 	{
-		SEXP names = PROTECT(Rf_getAttrib(Xi, R_NamesSymbol));
-		for (R_len_t i = 0; i < Rf_length(Xi); i++)
+		/* check if object is data.frame */
+		int doRecurse = TRUE;
+		if (!dfaslist)
 		{
-			(*ix)++;
-			C_fill_flat_names(ansNew, newNames, VECTOR_ELT(Xi, i), Rf_isNull(names) ? NA_STRING : STRING_ELT(names, i), xinfo, ix, ians);
+			SEXP df = PROTECT(Rf_ScalarString(Rf_mkChar("data.frame")));
+			if (C_matchClass(Xi, df))
+				doRecurse = FALSE;
+			UNPROTECT(1);
+		}
+
+		/* recurse deeper */
+		if (doRecurse)
+		{
+			SEXP names = PROTECT(Rf_getAttrib(Xi, R_NamesSymbol));
+			for (R_len_t i = 0; i < Rf_length(Xi); i++)
+				C_fill_flat_names(ansNew, newNames, VECTOR_ELT(Xi, i), Rf_isNull(names) ? NA_STRING : STRING_ELT(names, i), xinfo, ix, ians, maxnodes, dfaslist);
+			UNPROTECT(1);
 		}
-		UNPROTECT(1);
 	}
 }
 
 /* fill melted data.frame */
-static void C_fill_melt(SEXP ansFlat, SEXP ansNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians)
+static void C_fill_melt(SEXP ansFlat, SEXP ansNames, SEXP Xi, SEXP name, R_len_t (*xinfo)[3], R_len_t *ix, R_len_t *ians, R_len_t maxnodes, int dfaslist)
 {
 	// add name to vector
 	SET_STRING_ELT(ansNames, *ix, name);
+	(*ix)++; // increment counter
 
-	if (xinfo[*ix][0] == 1) // terminal nodes only
+	if (xinfo[*ix - 1][0] == 1) // terminal nodes only
 	{
 		SET_VECTOR_ELT(ansFlat, *ians, Xi);
 		(*ians)++;
 	}
 	else if (Rf_isVectorList(Xi))
 	{
-		SEXP names;
-		R_len_t m = Rf_length(Xi);
-		PROTECT_INDEX ipx;
-		PROTECT_WITH_INDEX(names = Rf_getAttrib(Xi, R_NamesSymbol), &ipx);
-		Rboolean noNames = Rf_isNull(names);
-
-		if (noNames)
-			REPROTECT(names = Rf_allocVector(STRSXP, m), ipx);
-
-		for (R_len_t i = 0; i < m; i++)
+		/* check if object is data.frame */
+		int doRecurse = TRUE;
+		if (!dfaslist)
 		{
-			(*ix)++;
-			// use counter for missing names
+			SEXP df = PROTECT(Rf_ScalarString(Rf_mkChar("data.frame")));
+			if (C_matchClass(Xi, df))
+				doRecurse = FALSE;
+			UNPROTECT(1);
+		}
+
+		/* recurse deeper */
+		if (doRecurse)
+		{
+			SEXP names;
+			R_len_t m = Rf_length(Xi);
+			PROTECT_INDEX ipx;
+			PROTECT_WITH_INDEX(names = Rf_getAttrib(Xi, R_NamesSymbol), &ipx);
+			Rboolean noNames = Rf_isNull(names);
+
 			if (noNames)
-				SET_STRING_ELT(names, i, C_int2char(i + 1));
+				REPROTECT(names = Rf_allocVector(STRSXP, m), ipx);
+
+			for (R_len_t i = 0; i < m; i++)
+			{
+				// use counter for missing names
+				if (noNames)
+					SET_STRING_ELT(names, i, C_int2char(i + 1));
 
-			// recurse further
-			C_fill_melt(ansFlat, ansNames, VECTOR_ELT(Xi, i), STRING_ELT(names, i), xinfo, ix, ians);
+				// recurse further
+				C_fill_melt(ansFlat, ansNames, VECTOR_ELT(Xi, i), STRING_ELT(names, i), xinfo, ix, ians, maxnodes, dfaslist);
+			}
+			UNPROTECT(1);
 		}
-		UNPROTECT(1);
 	}
 }
